"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const path_1 = require("path");
const command_2 = require("@salesforce/command");
const tmp = require("tmp");
const child_process_1 = require("child_process");
const fs_extra_1 = require("fs-extra");
const xml2js = require("xml2js");
command_2.core.Messages.importMessagesDirectory(path_1.join(__dirname, '..', '..', '..'));
const messages = command_2.core.Messages.loadMessages('sfdx-totalpackage', 'convert');
const isDirectory = (source) => fs_extra_1.lstatSync(source).isDirectory();
class Convert extends command_2.SfdxCommand {
    async run() {
        // this.project is guaranteed since requiresProject is true
        const config = await this.project.resolveProjectConfig();
        const xmlParser = new xml2js.Parser();
        let combinedPackageData;
        for (let elem of config['packageDirectories']) { // tslint:disable-line:prefer-const
            const path = elem['path'];
            // run sfdx force:source:convert
            const tmpDir = tmp.dirSync({ prefix: 'sfdx-totalpkg-' });
            let buildCmd = `sfdx force:source:convert -r ${path} -d ${tmpDir.name}`;
            if (this.flags.packagename) {
                buildCmd += ` -n ${this.flags.packagename}`;
            }
            this.ux.log(`exec: ${buildCmd}`);
            child_process_1.execSync(buildCmd);
            // copy files to output dir
            this.ux.log(`copying files from ${tmpDir.name} to ${this.flags.outputdir}`);
            fs_extra_1.readdirSync(tmpDir.name).forEach((localName) => {
                const fullPath = path_1.join(tmpDir.name, localName);
                if (!isDirectory(fullPath)) {
                    return;
                }
                fs_extra_1.copySync(fullPath, path_1.join(this.flags.outputdir, localName));
            });
            // merge package.xml with that from output dir
            const pkgFileContents = await fs_extra_1.readFile(path_1.join(tmpDir.name, 'package.xml'));
            const pkgData = await new Promise((resolve, reject) => {
                xmlParser.parseString(pkgFileContents, function (err, result) {
                    if (err) {
                        reject(err);
                    }
                    else {
                        resolve(result);
                    }
                });
            });
            if (!combinedPackageData) {
                combinedPackageData = pkgData;
            }
            else {
                // add to combined package data
                pkgData['Package']['types'].forEach((t) => {
                    // get type from combinedPackageData
                    const combinedType = combinedPackageData['Package']['types'].find((x) => x['name'][0] === t['name'][0]);
                    let combinedMembers = combinedType ? combinedType.members : [];
                    combinedMembers = combinedMembers.concat(t.members);
                    combinedMembers = Array.from(new Set(combinedMembers));
                    combinedType.members = combinedMembers;
                });
            }
        }
        // write combinedPackageData to package.xml in output dir
        const outputPackageXmlPath = path_1.join(this.flags.outputdir, 'package.xml');
        this.ux.log(`writing combined package.xml to ${outputPackageXmlPath}`);
        const builder = new xml2js.Builder({ standalone: undefined });
        const xml = builder.buildObject(combinedPackageData);
        await fs_extra_1.writeFile(outputPackageXmlPath, xml);
        return { pkgData: combinedPackageData };
    }
}
Convert.description = messages.getMessage('commandDescription');
Convert.examples = [
    '$ sfdx source:convert --outputdir mdapi --packagename mypkg'
];
Convert.flagsConfig = {
    // flag with a value (-n, --name=VALUE)
    outputdir: command_1.flags.string({ char: 'd', description: messages.getMessage('outputdirFlagDescription'), required: true }),
    packagename: command_1.flags.string({ char: 'n', description: messages.getMessage('packagenameFlagDescription') })
};
// Set this to true if your command requires a project workspace; 'requiresProject' is false by default
Convert.requiresProject = true;
exports.default = Convert;
//# sourceMappingURL=convert.js.map